import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/** An instance is a graph maintained as an adjacency matrix */ 
public class MatrixGraph implements Graph{
    public boolean[][] matrix; // adjacency matrix
    
    public int n;       // number of nodes
    public int m;       // number of edges

    /** A graph with n nodes numbers 0..n-1 and edges
     * given by edges.  */
    public MatrixGraph(int numNodes, Pair[] edges) {
        n= numNodes;
        m= edges.length;
        
        matrix= new boolean[n][n];
        for (Pair p : edges) {
            matrix[p.one][p.two]= true;
        }
    }
    
    /** Return the number of nodes in the graph */
	@Override
	public int numNodes() {
		return n;
	}
	
	/** Return a list of edges in the graph */
	public List<Pair> getEdges(){
		LinkedList<Pair> edges= new LinkedList<Pair>();
		for(int i=0; i<n; i++) {
			for(int j=0; j<n; j++) {
				if(matrix[i][j]) {
					edges.add(new Pair(i,j));
				}
			}
		}
		return edges;
	}
	
	/** Check whether an edge exists */
	public boolean hasEdge(int u, int v) {
		return matrix[u][v];
	}
    
   /** Return a list of neighbors of n.
    * Precondition: 0 <= n < number of nodes */
    public ArrayList<Integer> getNeighbors(int n) {
        ArrayList<Integer> s= new ArrayList<Integer>();
        for (int k= 0; k < matrix.length; k= k+1) {
            if (matrix[n][k]) s.add(k);
        }
        return s;
    }
    

    
    
    /** Print the graph.
     * Precondition: the graph has < 100 nodes */
    @Override
    public void printGraph() {
        System.out.println("Adjacency matrix");
        
        // print heading
        String heading= "     ";
        for (int c= 0; c < matrix.length; c= c+1) {
            heading= heading + make3(c) + " ";
        }
        System.out.println(heading);
        
        // Print graph
        for (int r= 0; r < matrix.length; r= r+1) {
            // print row r
            String row= make3(r) + "  ";
            for (int c= 0; c < matrix.length; c= c+1) {
                row= row + (matrix[r][c] ? "  1" : "  0") + " ";
            }
            System.out.println(row);
        }
        
    }
    
    /** = n in at least 3 chars, with leading blanks added if necessary. */
    public static String make3(int n) {
        if (0 <= n  &&  n < 10) return "  " + n;
        if (0 <= n  &&  n < 100) return " " + n;
        if (-10 < n  &&  n < 0)  return " " + n;
        return "" + n;
    }
    
    
    /** Build a graph and store it in game. */
    public static MatrixGraph exampleGraph1() {
        Pair[] edges= new Pair[] {
                new Pair(1, 2), new Pair(1, 5), new Pair(1, 7),
                new Pair(2, 5), new Pair(2, 3), 
                new Pair(3, 5),  
                new Pair(4, 5), new Pair(4, 6), 
                new Pair(7, 8)
        };
        return new MatrixGraph(9, edges);
        
    }
    
    /** Build a graph and store it in game. */
    public static MatrixGraph exampleGraph2() {
        Pair[] edges= new Pair[] {
                new Pair(1, 2), new Pair(1, 3), new Pair(1, 4),
                new Pair(2, 5), new Pair(2, 6), 
                new Pair(3, 6), new Pair(3, 7), 
                new Pair(4, 7), new Pair(4, 8)
        };
        return new MatrixGraph(9, edges);
        
    }



}

